<?php

/**
 * Zlick Payments Plugin
 *
 * @category  Zlick
 * @package   Zlick
 * @author    Arsalan Ahmad <arsalan@zlick.it>
 * @copyright Copyright (c) 2018 Zlick ltd (https://www.zlick.it)
 * @license   http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 * @link      https://www.zlick.it
 */

function zlickpay_widget_placeholder() {
	return '';
}

/**
 * Returns the smart prc widget.
 *
 * @param string $post_id The post id.
 *
 * @return string
 */
function zlickpay_widget_link_shortcode( $data ) {
	
	if (!is_single()) {
		return '';
	}

	$options_data = zlickpay_get_options_data();
	
		$post_id = get_the_ID();
		if (!empty($post_id)) {
			$zp_custom_fields = get_post_custom( $post_id );
			$article_price = '';
			if (zlickpay_cookie_is_valid_subscription() OR zlickpay_cookie_is_purchased_article(trim($post_id))) {
				return '';
			} else {
				$widget_text = get_option( 'zp_widget_text' );
				$zp_article_name = !empty($zp_custom_fields['zp_article_name']) ? $zp_custom_fields['zp_article_name'][0] : 'Article';
				$zp_enable_subscription = get_option( 'zp_enable_subscription', '' );
				$zp_article_price = zlickpay_sanitize_price($article_price);
				$zp_widget_text = str_replace('{price}',$article_price,$widget_text);
				// Documented in readme under "Technical" section.
				wp_enqueue_script( 'zlick-payments-sdk', 'https://cdn.zlick.it/zlick-paywall-element-1.1.0.js', '', '', true );
				wp_enqueue_script( ZLICK_PLUGIN_NAME , array('jquery', 'zlick-payments-sdk') , '1.0', true);
				wp_localize_script( ZLICK_PLUGIN_NAME , 'zlick_payments_ajax', 
					array(
						'ajax_url' => ZLICK_AJAX_URL,
						'client_token'=> $options_data['zp_client_token'],
						'user_id' => zlickpay_get_current_user_id(),
						'article_id' => $post_id,
						'zp_article_name' => $zp_article_name,
						'zp_ajax_nonce' => wp_create_nonce("zp-ajax-nonce")
					) 
				);
				ob_start();
				include ZLICK_PLUGIN_DIR . 'public/templates/zlick-widget.php';
				$widget = ob_get_contents();
				ob_end_clean();

				return $widget;
			}
		}

	return '';
}

/**
 * Register short codes.
 */
add_shortcode( 'zlick_payment_widget', 'zlickpay_widget_link_shortcode' );
add_shortcode('zp_placeholder', 'zlickpay_widget_placeholder');

/**
 * Register Ajax callback for logged in and anonymous users 
 */
add_action('wp_ajax_zp_authenticate_article','zlickpay_authenticate_article');
add_action('wp_ajax_nopriv_zp_authenticate_article','zlickpay_authenticate_article');

function zlickpay_shortcode_resource() {
	wp_register_script(ZLICK_PLUGIN_NAME, plugins_url("js/zlick-widget.js", __DIR__), array('jquery'), "1.0", false);
}

add_action( 'init', 'zlickpay_shortcode_resource' );

if ( !function_exists( 'zlickpay_cookie_register_article' ) ) {

	function zlickpay_cookie_register_article($article_id) {
		
		$secret = zlickpay_get_options_data('zp_client_secret');
		// $zp_article_cookie_content signed token
		$zp_article_cookie_content = isset($_COOKIE["zp_articles"]) ? $_COOKIE["zp_articles"] : '';

		if (empty($zp_article_cookie_content) || !isset($zp_article_cookie_content)) {
			$authed_articles = array();
		} else {
			$zp_article_cookie_content = zlickpay_validate_sign($secret, $zp_article_cookie_content);
			$authed_articles = explode(",", $zp_article_cookie_content);
		}

		
		$authed_articles[] = $article_id;
		$authed_articles = array_unique($authed_articles);

		$payload = join(",", $authed_articles);

		setcookie("zp_articles", zlickpay_jwt_sign($secret, $payload),  0, "/");
	}
}

if ( !function_exists( 'zlickpay_cookie_register_subscription' ) ) {
	function zlickpay_cookie_register_subscription($subscription_id) {
		$zp_options = zlickpay_get_options_data();
		$client_token = $zp_options['zp_client_token'];
		$client_secret = $zp_options['zp_client_secret'];

		$cookie_val = zlickpay_jwt_sign($client_secret, $client_token );

		setcookie("zp_subscription", $cookie_val, time() + 60 * 60 * 3, "/");
	}
}

if ( !function_exists( 'zlickpay_cookie_unregister_all' ) ) {
	function zlickpay_cookie_unregister_all()
	{
		setcookie("zp_subscription", "",  time() - 10000, "/");
		setcookie("zp_articles", "",  time() - 10000, "/");
	}
}

if ( !function_exists( 'zlickpay_authenticate_article' ) ) {

	/**
	 * @return false|int|null
	 */
	function zlickpay_authenticate_article(){
		
		// Check for nonce security      
		if ( !isset($_POST['zp_nonce']) && ! wp_verify_nonce( $_POST['zp_nonce'], 'zp-ajax-nonce' ) ) {
			die ( 'nonce could not be verified!!!');
		}

		$zp_options = zlickpay_get_options_data();
		$client_token = $zp_options['zp_client_token'];
		$client_secret = $zp_options['zp_client_secret'];

		if (!zlickpay_validate_sign( $client_secret, sanitize_text_field($_POST['zp_signed']))) {
			exit("\nThe signature is NOT valid.");
		}
		
		$article_id = intval(sanitize_text_field($_POST['zp_article_id']));
		
		$signedPayload = json_decode(zlickpay_get_jwt_payload(sanitize_text_field($_POST['zp_signed'])), true);

		if (isset($signedPayload["transaction"]["hasAccess"]) && $signedPayload["transaction"]["hasAccess"] === true) {
			$article_id = intval(sanitize_text_field($signedPayload["transaction"]["productId"]));
			error_log("\nRegistering article".$article_id);
			zlickpay_cookie_register_article($article_id);
		} elseif (isset($signedPayload["subscription"]["hasAccess"]) && $signedPayload["subscription"]["hasAccess"] === true) {
			$subscription_id = intval(sanitize_text_field($signedPayload["subscription"]["subscriptionId"]));
			error_log("\nRegistering subscription".$subscription_id);
			zlickpay_cookie_register_subscription($subscription_id);
		} else {
			zlickpay_cookie_unregister_all();
		}

		wp_die("reload");

		return 'Done. Reload the page now';
	}
}

if ( !function_exists( 'zlickpay_cookie_is_valid_subscription' ) ) {
	
	function zlickpay_cookie_is_valid_subscription() {
		// $cookie_val contains jwt signed string that is validated before further use
		$cookie_val = isset($_COOKIE["zp_subscription"]) ? $_COOKIE["zp_subscription"] : "";
		$secret = zlickpay_get_options_data( 'zp_client_secret' );
		
		if ($cookie_val === "" ) {
			return false;
		}
		if (!zlickpay_validate_sign($secret, $cookie_val)) {
			return false;
		}

		$payload = zlickpay_get_jwt_payload($cookie_val);

		if ($payload === zlickpay_get_options_data( 'zp_client_token' )) {
			return true;
		}
		return false;
	}
}

if ( !function_exists( 'zlickpay_cookie_is_purchased_article' ) ) {
	
	function zlickpay_cookie_is_purchased_article($article_id) {
		// $cookie_val contains jwt signed string that is validated before further use
		$cookie_val = isset($_COOKIE["zp_articles"]) ? $_COOKIE["zp_articles"] : "";
		$secret = zlickpay_get_options_data( 'zp_client_secret' );
		// @todo validate signed token || !zlickpay_validate_sign($secret, $cookie_val)
		if ( $cookie_val === "" ) {
			return false;
		}
		if (!zlickpay_validate_sign($secret, $cookie_val)) {
			return false;
		}

		$payload = zlickpay_get_jwt_payload($cookie_val);
		$purchased_articles = explode(",", $payload);
		echo "<script>console.log('Checking if this article is purchased: "  . $article_id . "');</script>";
		echo "<script>console.log('Purchased articles: " . json_encode($purchased_articles) . "');</script>";


		return in_array($article_id, $purchased_articles);
	}
}

if ( !function_exists( 'zlickpay_find_number_paragraphs' ) ) {
	/**
	 * Calculate previewable paragrpahs
	 * 
 	 * @param $content
 	 * @param $para_count
	 *
	 * @return false|string
	 */
	function zlickpay_find_number_paragraphs($content, $para_count) {
		$para_i = 0;
		$char_count = 0;

		do {
			$para_i++;
			$ti = strpos($content, '</p>', $char_count + 1);
			if ($ti === FALSE)  {
			break;
			}
			$char_count = $ti;
		} while($para_i < $para_count);

		return $char_count;
	}
}

if ( !function_exists( 'zlickpay_limit_content' ) ) {
	/**
	 * Limit Content renerding to previewable_paragraph option
	 * 
	 * @param $content
	 *
	 * @return false|string
	 */
	function zlickpay_limit_content($content) {

		// Makes it compatible with both gutenberg and classic editors
		if (!strpos($content, "</p>")) {
			$content = wpautop( $content );
			remove_filter( 'the_content', 'wpautop' );
		}

		$post_id = get_the_ID();
		$zp_custom_fields = get_post_custom( $post_id );
		if (!isset($zp_custom_fields['zp_is_paid']) || $zp_custom_fields['zp_is_paid'][0] != 'paid') {
			return $content;
		}

		if (get_post_type() == ZLICK_POST_TYPE_POST ) {
			$needle = <<<TMP
			<!-- wp:shortcode -->
			[zp_placeholder]
			<!-- /wp:shortcode -->
TMP;
			
			if(!zlickpay_cookie_is_valid_subscription() and !zlickpay_cookie_is_purchased_article(get_the_ID())) {
				$content_length = strpos($content, $needle);
				if ($content_length === false) {
					$para_count = zlickpay_get_options_data( 'zp_previewable_paras' );

					$content_length = zlickpay_find_number_paragraphs($content, $para_count);
				}
				$content = substr( $content, 0, $content_length );
				$content .= do_shortcode( "[zlick_payment_widget post_id=" . get_the_ID() . "]" );
			}
		}

		return $content;
	}
}

add_filter( "the_content", "zlickpay_limit_content", -1);

if ( !function_exists( 'zlickpay_get_options_data' ) ) {
	/**
	 * Retrieve saved configurations option by key|all
	 *
	 * @return array
	 */
	function zlickpay_get_options_data( $key = 'all') {
		$zlick_options = get_option( 'zlick_payments_settings', [] );
		
		if(empty($zlick_options)){
			return;
		}

		return $key == 'all' ? $zlick_options : $zlick_options[$key];
	}
}

if ( !function_exists( 'zlickpay_sanitize_price' ) ) {
	/**
	 * Round the Price upto 2 decimal values
	 *
	 * @return array
	 */
	function zlickpay_sanitize_price($price) {
		$iprice = (float) $price;
		$iprice = round($iprice * 100);
		return $iprice;
	}
}